@@ -234,9 +234,10 @@ class EventEmitter:
         if event not in self._listeners:
             return 0
 
-        listeners = self._listeners[event].copy()  # Copy to allow modification
+        listeners: list[EventListener] = self._listeners[event].copy()
+        # Copy to allow modification during iteration
         called_count = 0
-        listeners_to_remove = []
+        listeners_to_remove: list[EventListener] = []
 
         for listener in listeners:
             try:
@@ -398,11 +399,9 @@ class EventEmitter:
 
         try:
             if timeout is not None:
-                args_tuple, kwargs_dict = await asyncio.wait_for(
-                    future, timeout=timeout
-                )
+                args_tuple, _ = await asyncio.wait_for(future, timeout=timeout)
             else:
-                args_tuple, kwargs_dict = await future
+                args_tuple, _ = await future
 
             # Return just args for simplicity (most common case)
             return args_tuple
