@@ -16,7 +16,7 @@ import json
 import logging
 import uuid
 from collections.abc import Callable
-from typing import TYPE_CHECKING, Any, cast
+from typing import TYPE_CHECKING, Any, Literal, cast
 
 from awscrt import mqtt
 from awscrt.exceptions import AwsCrtError
@@ -31,6 +31,7 @@ from ..exceptions import (
     MqttPublishError,
     TokenRefreshError,
 )
+from ..unit_system import set_unit_system
 
 if TYPE_CHECKING:
     from ..models import (
@@ -135,6 +136,7 @@ class NavienMqttClient(EventEmitter):
         self,
         auth_client: NavienAuthClient,
         config: MqttConnectionConfig | None = None,
+        unit_system: Literal["metric", "imperial"] | None = None,
     ):
         """
         Initialize the MQTT client.
@@ -142,10 +144,14 @@ class NavienMqttClient(EventEmitter):
         Args:
             auth_client: Authentication client with valid tokens
             config: Optional connection configuration
+            unit_system: Preferred unit system:
+                - "metric": Celsius, LPM, Liters
+                - "imperial": Fahrenheit, GPM, Gallons
+                - None: Auto-detect from device (default)
 
         Raises:
-            MqttCredentialsError: If auth client is not authenticated, tokens
-                are stale/expired, or AWS credentials are not available
+            MqttCredentialsError: If auth client is not authenticated or AWS
+                credentials are not available
         """
         if not auth_client.is_authenticated:
             raise MqttCredentialsError(
@@ -153,11 +159,12 @@ class NavienMqttClient(EventEmitter):
                 "creating MQTT client. Call auth_client.sign_in() first."
             )
 
-        if not auth_client.has_valid_tokens:
-            raise MqttCredentialsError(
-                "Tokens are stale/expired. "
-                "Call ensure_valid_token() or re_authenticate() first."
-            )
+        # Token validity is checked in connect() which also refreshes stale
+        # tokens automatically. This allows creating MQTT clients with
+        # restored tokens that may have expired between sessions. Token
+        # validation and refresh are deferred until connect() is called; if
+        # connect() is never called, tokens are not revalidated/refreshed
+        # and no MQTT connection is established.
 
         if not auth_client.current_tokens:
             raise MqttCredentialsError("No tokens available from auth client")
@@ -172,7 +179,12 @@ class NavienMqttClient(EventEmitter):
         # Initialize EventEmitter
         super().__init__()
 
+        # Set unit system preference if provided
+        if unit_system is not None:
+            set_unit_system(unit_system)
+
         self._auth_client = auth_client
+        self._unit_system: Literal["metric", "imperial"] | None = unit_system
         self.config = config or MqttConnectionConfig()
 
         # Session tracking
@@ -552,6 +564,7 @@ class NavienMqttClient(EventEmitter):
                     event_emitter=self,
                     schedule_coroutine=self._schedule_coroutine,
                     device_info_cache=device_info_cache,
+                    unit_system=self._unit_system,
                 )
 
                 # Initialize device controller with cache
@@ -657,7 +670,9 @@ class NavienMqttClient(EventEmitter):
 
     def _create_credentials_provider(self) -> Any:
         """Create AWS credentials provider from auth tokens."""
-        from awscrt.auth import AwsCredentialsProvider
+        from awscrt.auth import (
+            AwsCredentialsProvider,
+        )
 
         # Get current tokens from auth client
         auth_tokens = self._auth_client.current_tokens
